<?php

namespace App\Http\Controllers;
use Illuminate\Support\Str;
use App\Models\GlobalFunction;
use App\Models\Album;
use Illuminate\Http\Request;
use App\Models\Artist;
use Illuminate\Validation\ValidationException;

class ArtistController extends Controller
{
  public function create(Request $request)
{
    try {
        $validated = $request->validate([
            'name' => 'required|string|max:255',
            'description' => 'nullable|string',
            'background' => 'nullable|string',
            'personality' => 'nullable|string',
            'appearance' => 'nullable|string',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048', // Validate the image
        ]);

        // Initialize the image path variable
        $imagePath = null;

        // Handle image upload if provided
        if ($request->hasFile('image')) {
            $imagePath = GlobalFunction::saveFileAndGivePath($request->file('image'), "artists");
        }

        // Create the artist
        $artist = Artist::create([
            'name' => $validated['name'],
            'description' => $validated['description'] ?? null,
            'background_info' => $validated['background'] ?? null,
            'personality_traits' => $validated['personality'] ?? null,
            'appearance_details' => $validated['appearance'] ?? null,
            'image' => $imagePath, // Save the image path
        ]);

        return response()->json(['message' => 'Artist created successfully', 'data' => $artist], 201);
    } catch (ValidationException $e) {
        return response()->json(['message' => 'Validation failed', 'errors' => $e->errors()], 422);
    } catch (\Exception $e) {
        return response()->json(['message' => 'Failed to create artist', 'error' => $e->getMessage()], 500);
    }
}


  public function updateArtist(Request $request)
{
    try {
        // Validate the input
        $validated = $request->validate([
            'id' => 'required|integer|exists:artists,id',
            'name' => 'required|string|max:255',
            'description' => 'nullable|string|max:1000',
            'background_info' => 'nullable|string|max:1000',
            'personality_traits' => 'nullable|string|max:1000',
            'appearance_details' => 'nullable|string|max:1000',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048', // Validate image
        ]);

        // Find the artist
        $artist = Artist::findOrFail($validated['id']);

        // Update artist fields
        $artist->name = $validated['name'];
        $artist->description = $validated['description'] ?? $artist->description;
        $artist->background_info = $validated['background_info'] ?? $artist->background_info;
        $artist->personality_traits = $validated['personality_traits'] ?? $artist->personality_traits;
        $artist->appearance_details = $validated['appearance_details'] ?? $artist->appearance_details;

        // Handle image upload if provided
        if ($request->hasFile('image')) {
            // Delete the old image if it exists
            if ($artist->image) {
                GlobalFunction::deleteFile($artist->image); // Ensure this function handles missing files
            }

            // Save the new image
            $artist->image = GlobalFunction::saveFileAndGivePath($request->file('image'), "artists");
        }

        // Save the artist
        $artist->save();

        return response()->json(['success' => true, 'message' => 'Artist updated successfully.'], 200);
    } catch (ValidationException $e) {
        // Handle validation errors
        return response()->json([
            'success' => false,
            'message' => 'Validation failed.',
            'errors' => $e->errors(),
        ], 422);
    } catch (\Exception $e) {
        // Log the error for debugging
        \Log::error('Error updating artist: ' . $e->getMessage());

        return response()->json([
            'success' => false,
            'message' => 'Failed to update artist.',
            'error' => $e->getMessage(),
        ], 500);
    }
}


    public function fetch($id)
    {
        try {
            $artist = Artist::findOrFail($id);

            return response()->json(['data' => $artist], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Failed to fetch artist', 'error' => $e->getMessage()], 500);
        }
    }

    public function delete($id)
    {
        try {
            $artist = Artist::findOrFail($id);
            $artist->delete();

            return response()->json(['message' => 'Artist deleted successfully'], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Failed to delete artist', 'error' => $e->getMessage()], 500);
        }
    }

    public function listAll()
    {
        try {
            $artists = Artist::paginate(10);

            return response()->json($artists, 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Failed to retrieve artists', 'error' => $e->getMessage()], 500);
        }
    }

    public function search(Request $request)
    {
        try {
            $validated = $request->validate([
                'query' => 'required|string|max:255',
            ]);

            $query = $validated['query'];

            $artists = Artist::where('name', 'like', "%$query%")
                ->orWhere('description', 'like', "%$query%")
                ->orWhere('background_info', 'like', "%$query%")
                ->paginate(10);

            return response()->json($artists, 200);
        } catch (ValidationException $e) {
            return response()->json(['message' => 'Validation failed', 'errors' => $e->errors()], 422);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Failed to search artists', 'error' => $e->getMessage()], 500);
        }
    }

    public function indexnew(Request $request)
    {
        if ($request->ajax()) {
            $query = Artist::query();
            $totalData = $query->count();

            if ($search = $request->input('search.value')) {
                $query->where('name', 'LIKE', "%{$search}%");
            }

            $artists = $query->offset($request->input('start'))
                             ->limit($request->input('length'))
                             ->get();

            return response()->json([
    "draw" => intval($request->input('draw')),
    "recordsTotal" => $totalData,
    "recordsFiltered" => $totalData,
    "data" => $artists->map(function ($artist) {
        return [
            'name' => $artist->name ?: 'N/A',
            'image' => $artist->image ? '<img src="' . GlobalFunction::createMediaUrl($artist->image) . '" alt="Image" width="50" height="50">' : 'N/A',
            'description' => $artist->description ? Str::limit($artist->description, 50) : 'N/A',
            'background_info' => $artist->background_info ? Str::limit($artist->background_info, 50) : 'N/A',
            'personality_traits' => $artist->personality_traits ? Str::limit($artist->personality_traits, 50) : 'N/A',
            'appearance_details' => $artist->appearance_details ? Str::limit($artist->appearance_details, 50) : 'N/A',
            'action' => '<button class="btn btn-success edit" data-id="' . $artist->id . '">Edit</button>
                         <button class="btn btn-danger delete" data-id="' . $artist->id . '">Delete</button>',
        ];
    }),
]);
        }

        return view('artists.index');
    }

    public function storenew(Request $request)
    {
        $data = $request->validate([
            'name' => 'required|string|max:255',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'description' => 'required|string',
            'background_info' => 'nullable|string',
            'personality_traits' => 'nullable|string',
            'appearance_details' => 'nullable|string',
        ]);

        // Handle image upload if provided
        if ($request->hasFile('image')) {
            $data['image'] = GlobalFunction::saveFileAndGivePath($request->file('image'), "artists");
        }
        Artist::create($data);

        return response()->json(['success' => true, 'message' => 'Artist added successfully.']);
    }

    public function editnew($id)
    {
        $artist = Artist::find($id);

        if (!$artist) {
            return response()->json(['success' => false, 'message' => 'Artist not found'], 404);
        }

        return response()->json($artist);
    }

  public function updatenew(Request $request)
{
    try {
        // Validate the input
        $data = $request->validate([
            'id' => 'required|integer|exists:artists,id',
            'name' => 'required|string|max:255',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048', // Validate image
            'description' => 'nullable|string|max:1000',
            'background_info' => 'nullable|string|max:1000',
            'personality_traits' => 'nullable|string|max:1000',
            'appearance_details' => 'nullable|string|max:1000',
        ]);

        // Find the artist
        $artist = Artist::findOrFail($data['id']);

        // Update artist fields
        $artist->name = $data['name'];
        $artist->description = $data['description'] ?? $artist->description;
        $artist->background_info = $data['background_info'] ?? $artist->background_info;
        $artist->personality_traits = $data['personality_traits'] ?? $artist->personality_traits;
        $artist->appearance_details = $data['appearance_details'] ?? $artist->appearance_details;

        // Handle image upload if provided
        if ($request->hasFile('image')) {
            // Delete the old image if it exists
            if ($artist->image) {
                GlobalFunction::deleteFile($artist->image); // Ensure deleteFile handles missing files gracefully
            }

            // Save the new image and update the artist's image field
            $artist->image = GlobalFunction::saveFileAndGivePath($request->file('image'), "artists");
        }

        // Save the artist record
        $artist->save();

        return response()->json([
            'success' => true,
            'message' => 'Artist updated successfully.',
            'data' => $artist,
        ], 200);
    } catch (ValidationException $e) {
        // Handle validation errors
        return response()->json([
            'success' => false,
            'message' => 'Validation failed.',
            'errors' => $e->errors(),
        ], 422);
    } catch (\Exception $e) {
        // Log the error for debugging
        \Log::error('Error updating artist: ' . $e->getMessage());

        return response()->json([
            'success' => false,
            'message' => 'Failed to update artist.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

    public function destroynew(Request $request)
    {
        $id = $request->input('id');
        $artist = Artist::find($id);

        if (!$artist) {
            return response()->json(['success' => false, 'message' => 'Artist not found'], 404);
        }

        $artist->delete();

        return response()->json(['success' => true, 'message' => 'Artist deleted successfully.']);
    }
    public function getAlbumsByArtist($artist_id)
{
    try {
        // Validate if artist exists
        $artist = Artist::find($artist_id);
        if (!$artist) {
            return response()->json([
                'status' => false,
                'message' => 'Artist not found.',
                'data' => []
            ], 404);
        }

        // Fetch albums belonging to the artist
        $albums = Album::where('artist_id', $artist_id)->get();

        // Check if the artist has any albums
        if ($albums->isEmpty()) {
            return response()->json([
                'status' => false,
                'message' => 'No albums found for this artist.',
                'data' => []
            ], 200);
        }

        // Format the response data
        return response()->json([
            'status' => true,
            'message' => 'Albums retrieved successfully.',
            'data' => $albums->map(function ($album) {
                return [
                    'album_id' => $album->id,
                    'title' => $album->title,
                    'genre_id' => $album->genre_id,
                    'image' => $album->image ? $album->image : 'No Image',
                    'created_at' => $album->created_at,
                    'updated_at' => $album->updated_at,
                    'category_option' => $album->category_option, // ✅ Add a type field if needed
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch albums.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

}
