<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\GuidedSessionTemplate;
use App\Models\Music;
use App\Models\Podcast;
use App\Models\VibeCheck;
use App\Models\UserSessionLog;
use App\Models\User;
use App\Models\StrainType;
use App\Services\NarrationService;

class DynamicSessionController extends Controller
{
    public function launch(Request $request, $id)
    {
        $userId = $request->input('user_id');
        $user = User::find($userId);
        $userName = $user?->fullname ?? 'friend';

        $template = GuidedSessionTemplate::findOrFail($id);

        // 1. Vibe + Time Tag
        $vibe = VibeCheck::where('user_id', $userId)->latest()->first()?->vibe_type ?? null;
        $hour = now()->hour;
        $timeTag = $hour < 12 ? 'morning' : ($hour < 18 ? 'afternoon' : 'night');

        // 2. Tags
        $allTags = array_filter(array_unique(array_merge(
            $template->goal_tags ?? [],
            $template->mood_tags ?? [],
            $template->sound_tags ?? [],
            $template->cannabis_tags ?? [],
            $template->use_case_tags ?? [],
            [$template->time_of_day, $vibe, $timeTag]
        )));

        $cannabisTags = array_filter($allTags, function ($tag) {
            return in_array(strtolower($tag), ['indica', 'sativa', 'hybrid', 'high-cbd', 'none', 'cbd-only']);
        });

        // 3. EQ from dominant cannabis tag
        $eqProfile = null;
        if (!empty($cannabisTags)) {
$dominant = !empty($cannabisTags) ? ucfirst(strtolower(array_values($cannabisTags)[0])) : null;
            $strainEq = StrainType::where('strain_types', $dominant)->first();
            $eqProfile = $strainEq?->eq ? json_decode($strainEq->eq) : null;
        }

        // 4. Music (randomized with filtered tags)
        $music = Music::where('active', 1)
            ->where(function ($query) use ($allTags, $cannabisTags) {
                $query->where(function ($q) use ($cannabisTags) {
                    foreach ($cannabisTags as $tag) {
                        $q->orWhere('tag', 'like', "%$tag%");
                    }
                });
                $query->where(function ($q) use ($allTags) {
                    foreach ($allTags as $tag) {
                        $q->orWhere('tag', 'like', "%$tag%");
                    }
                });
            })
            ->inRandomOrder()
            ->limit(6)
            ->get();

        // 5. Music Narration
        $lastTrackTitle = null;
        foreach ($music as $track) {
            $trackTags = explode(',', $track->tag);
            $track->narration = NarrationService::generate('music', [
                'track_title' => $track->title,
                'tags' => implode(', ', $trackTags),
                'previous_track_title' => $lastTrackTitle,
                'user_name' => $userName,
                'cannabis_tag' => $cannabisTags[0] ?? null,
            ]);
            $lastTrackTitle = $track->title;
        }

        // 6. Single activity returned
        $activity = [
            'title' => $template->title,
            'description' => $template->description,
            'duration' => 0, // Optional: sum track durations if available
            'tags_used' => $allTags,
            'eq_profile' => $eqProfile,
            'music' => $music,
            'podcasts' => [] // Disabled
        ];

        // 7. Narrations
        $intro = NarrationService::generate('intro', [
            'session_title' => $template->title,
            'vibe' => $vibe,
            'time_tag' => $timeTag,
            'goals' => $template->goal_tags,
            'user_name' => $userName,
            'first_track_title' => $music[0]->title ?? null,
            'cannabis_tag' => $cannabisTags[0] ?? null,
        ]);

        $outro = NarrationService::generate('outro', [
            'session_title' => $template->title,
            'user_name' => $userName,
        ]);

        // 8. Log
        $log = UserSessionLog::create([
            'user_id' => $userId,
            'guided_session_template_id' => $template->id,
            'vibe_type' => $vibe,
            'time_of_day' => $timeTag,
            'tags_used' => $allTags,
            'total_duration' => 0, // Optional: sum durations
            'launched_at' => now()
        ]);

        // 9. Response
        return response()->json([
            'status' => true,
            'session' => [
                'session_log_id' => $log->id,
                'title' => $template->title,
                'description' => $template->description,
                'vibe_used' => $vibe,
                'time_tag' => $timeTag,
                'tags_combined' => $allTags,
                'total_duration' => 0, // Update if using track durations
                'intro_narration' => $intro,
                'outro_narration' => $outro,
                'activities' => [$activity]
            ]
        ]);
    }
}
