<?php

namespace App\Http\Controllers;

use App\Models\Genre;
use Illuminate\Http\Request;

class GenreController extends Controller
{
    /**
     * Create a new Genre (old create method).
     */
    public function create(Request $request)
    {
        $validated = $request->validate([
            'name'        => 'required|string|max:255|unique:genres,name',
            'description' => 'nullable|string|max:1000',
            'parent_id'   => 'nullable|exists:genres,id',
            'type'        => 'required|in:indica,sativa,hybrid,meditation', 
        ]);

        $genre = Genre::create([
            'name'        => $validated['name'],
            'description' => $validated['description'] ?? null,
            'parent_id'   => $validated['parent_id'] ?? null,
            'type'        => $validated['type'],
        ]);

        return response()->json([
            'message' => 'Genre created successfully',
            'data' => $genre
        ], 201);
    }

    /**
     * Update an existing Genre (old update method).
     */
    public function update(Request $request, $id)
    {
        $genre = Genre::findOrFail($id);

        $validated = $request->validate([
            'name'        => 'nullable|string|max:255|unique:genres,name,' . $id,
            'description' => 'nullable|string|max:1000',
            'parent_id'   => 'nullable|exists:genres,id',
            'type'        => 'required|in:indica,sativa,hybrid,meditation',
        ]);

        // Only update the fields provided
        $genre->update($validated);

        return response()->json([
            'message' => 'Genre updated successfully',
            'data' => $genre
        ], 200);
    }

    public function delete($id)
    {
        $genre = Genre::findOrFail($id);

        if (Genre::where('parent_id', $id)->exists()) {
            return response()->json(['message' => 'Cannot delete a genre with sub-genres'], 400);
        }

        $genre->delete();

        return response()->json(['message' => 'Genre deleted successfully'], 200);
    }

    /**
     * List all genres that have a parent (sub-genres).
     */
    public function listAll()
    {
        $genres = Genre::with('subGenres')->whereNull('parent_id')->get();

        return response()->json(['data' => $genres], 200);
    }

    /**
     * Fetch a single genre by ID (including subGenres).
     */
    public function fetch($id)
    {
        $genre = Genre::with('subGenres')->findOrFail($id);

        return response()->json(['data' => $genre], 200);
    }

    /**
     * DataTables-compatible index for AJAX calls.
     */
    public function indexnew(Request $request)
    {
        if ($request->ajax()) {
            $query = Genre::with('parent');
            $totalData = $query->count();

            if ($search = $request->input('search.value')) {
                $query->where('name', 'LIKE', "%{$search}%");
            }

            $genres = $query->offset($request->input('start'))
                            ->limit($request->input('length'))
                            ->get();

            return response()->json([
                "draw" => intval($request->input('draw')),
                "recordsTotal" => $totalData,
                "recordsFiltered" => $totalData, 
                "data" => $genres->map(function ($genre) {
                    return [
                        'name' => $genre->name,
                        'description' => $genre->description,
                        'parent' => $genre->parent->name ?? 'N/A',
                        // If you want to show 'type' in the table:
                        'type' => $genre->type ?? 'N/A',
                        'action' => '<button class="btn btn-success edit" data-id="'.$genre->id.'">Edit</button>
                                     <button class="btn btn-danger delete" data-id="'.$genre->id.'">Delete</button>',
                    ];
                }),
            ]);
        }

        $parents = Genre::get();
        return view('genres.index', compact('parents'));
    }

    /**
     * Store a new Genre (newer method, presumably AJAX).
     */
    public function storenew(Request $request)
    {
        $data = $request->validate([
            'name'        => 'required|string|max:255',
            'description' => 'nullable|string',
            'parent_id'   => 'nullable|integer|exists:genres,id',
            'type'        => 'required|in:indica,sativa,hybrid,meditation',
        ]);
    
        $genre = Genre::create($data);
    
        return response()->json([
            'success' => true,
            'message' => 'Genre added successfully.',
            'data' => $genre, 
        ]);
    }

    /**
     * Return data for editing (new method).
     */
    public function editnew($id)
    {
        $genre = Genre::findOrFail($id);
        $parents = Genre::where('id','!=',$id)->get();

        return response()->json([
            'genre'   => $genre,
            'parents' => $parents
        ]);
    }

    /**
     * Update a genre (new method).
     */
    public function updatenew(Request $request)
    {
        $data = $request->validate([
            'id'          => 'required|integer|exists:genres,id',
            'name'        => 'required|string|max:255',
            'description' => 'nullable|string',
            'parent_id'   => 'nullable|integer|exists:genres,id',
            'type'        => 'required|in:indica,sativa,hybrid,meditation',
        ]);

        $genre = Genre::findOrFail($data['id']);
        $genre->update($data);

        return response()->json(['success' => true, 'message' => 'Genre updated successfully.']);
    }

    /**
     * Delete a genre (new method).
     */
    public function destroynew(Request $request)
    {
        $id = $request->input('id');
        $genre = Genre::findOrFail($id);
        $genre->delete();

        return response()->json(['success' => true, 'message' => 'Genre deleted successfully.']);
    }
}
