<?php

namespace App\Http\Controllers;

use App\Models\Music;
use App\Models\Playlist;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Models\User;
use App\Models\GlobalFunction;

class PlaylistController extends Controller
{
    public function createPlaylist(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'playlist_name' => 'required|string|max:255',
                'user_id' => 'required|exists:users,id',
                'image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            ]);

            $file = $request->file('image');
            $imagePath = GlobalFunction::saveFileAndGivePath($file, "playlist-images");

            $playlist = Playlist::create([
                'playlist_name' => $validatedData['playlist_name'],
                'user_id' => $validatedData['user_id'],
                'image' => $imagePath,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Playlist created successfully.',
                'data' => $playlist,
            ], 201);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to create playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

   public function addMusicToPlaylist(Request $request)
{
    try {
        $validatedData = $request->validate([
            'playlist_id' => 'required|exists:playlists,id',
            'music_id' => 'required|exists:music,id',
        ]);

        $playlist = Playlist::find($validatedData['playlist_id']);

        // Check if the music already exists in the playlist
        if ($playlist->music()->where('music_id', $validatedData['music_id'])->exists()) {
            return response()->json([
                'status' => false,
                'message' => 'Music already exists in the playlist.',
            ], 200); // HTTP 400 for a bad request
        }

        // Attach the music to the playlist
        $playlist->music()->attach($validatedData['music_id']);

        return response()->json([
            'status' => true,
            'message' => 'Music added to playlist successfully.',
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to add music to playlist.',
            'error' => $e->getMessage(),
        ], 500);
    }
}



    public function playTrack(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'playlist_id' => 'required|exists:playlists,id',
                'music_id' => 'required|exists:music,id',
            ]);

            $playlist = Playlist::find($validatedData['playlist_id']);
            if (!$playlist->music()->where('id', $validatedData['music_id'])->exists()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Track not found in playlist.',
                ], 404);
            }

            $music = Music::find($validatedData['music_id']);
            $music->increment('plays');

            DB::table('music_play_logs')->insert([
                'music_id' => $music->id,
                'played_at' => now(),
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Track played successfully.',
                'data' => $music,
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to play track.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }


    public function deletePlaylist($id)
    {
        try {
            $playlist = Playlist::findOrFail($id);
            $playlist->delete();

            return response()->json([
                'status' => true,
                'message' => 'Playlist deleted successfully.',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to delete playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function likePlaylist($id)
    {
        try {
            $playlist = Playlist::findOrFail($id);
            $playlist->increment('likes');

            return response()->json([
                'status' => true,
                'message' => 'Playlist liked successfully.',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to like playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

   public function index(Request $request)
{
    if ($request->ajax()) {
        $query = Playlist::with('user');
        $totalData = $query->count();

        // Apply search filter
        if ($search = $request->input('search.value')) {
            $query->where('playlist_name', 'LIKE', "%{$search}%")
                  ->orWhereHas('user', fn($q) => $q->where('fullname', 'LIKE', "%{$search}%"));
        }

        $totalFiltered = $query->count(); // Recalculate after search filter

        // Handle pagination parameters with defaults
        $start = $request->input('start', 0);
        $length = $request->input('length', 10);

        // Get paginated data
        $playlists = $query->offset($start)
                           ->limit($length)
                           ->get();

        return response()->json([
            "draw" => intval($request->input('draw', 1)),
            "recordsTotal" => $totalData,
            "recordsFiltered" => $totalFiltered,
            "data" => $playlists->map(function ($playlist) {
                return [
                    'playlist_name' => $playlist->playlist_name,
                    'user' => $playlist->user->fullname ?? 'N/A',
                    'image' => $playlist->image ? GlobalFunction::createMediaUrl($playlist->image) : 'No Image',
                    'playlisttype' => $playlist->playlisttype ?? 0,
                    'likes' => $playlist->likes ?? 'N/A',
                    'action' => '<button class="btn btn-success edit" data-id="' . e($playlist->id) . '">Edit</button>
                                 <button class="btn btn-danger delete" data-id="' . e($playlist->id) . '">Delete</button>',
                ];
            }),
        ]);
    }

    $users = User::all();
    return view('playlists.index', compact('users'));
}
    /**
     * Store a newly created playlist.
     */
    public function store(Request $request)
    {
        $data = $request->validate([
            'playlist_name' => 'required|string|max:255',
            'user_id' => 'required|integer',
            'image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            'playlisttype' => 'nullable|integer|in:0,1',
        ]);
            $data['playlisttype'] = $request->input('playlisttype', 0); // default to community

        if ($request->hasFile('image')) {
            $file = $request->file('image');
            $imagePath = GlobalFunction::saveFileAndGivePath($file, "playlist-images");
            $data['image'] = $imagePath;

        }

        Playlist::create($data);

        return response()->json(['success' => true, 'message' => 'Playlist added successfully.']);
    }

    /**
     * Show the form for editing the specified playlist.
     */
    public function edit($id)
    {
        $playlist = Playlist::find($id);

        if (!$playlist) {
            return response()->json(['success' => false, 'message' => 'Playlist not found'], 404);
        }

        return response()->json([
            'success' => true,
            'playlist_name' => $playlist->playlist_name,
            'user_id' => $playlist->user_id, 
            'image' => GlobalFunction::createMediaUrl($playlist->image),
                'playlisttype' => $playlist->playlisttype ?? 0, // ✅ Add this

        ]);
    }


/**
 * Get all playlists with likes count and songs count.
 */
public function getPlaylistsWithDetails()
{
    try {
        $playlists = Playlist::with('user') // Include user relationship
            ->select('playlists.*', DB::raw('COUNT(playlist_music.id) as song_count'), DB::raw('COALESCE(playlists.likes, 0) as likes'))
            ->leftJoin('playlist_music', 'playlists.id', '=', 'playlist_music.playlist_id') // Join with playlist_music
            ->groupBy('playlists.id', 'playlists.playlist_name', 'playlists.user_id', 'playlists.image', 'playlists.likes', 'playlists.created_at', 'playlists.updated_at')
            ->get();

        return response()->json([
            'status' => true,
            'message' => 'Playlists fetched successfully.',
            'data' => $playlists->map(function ($playlist) {
                return [
                    'id' => $playlist->id,
                    'playlist_name' => $playlist->playlist_name,
                    'user' => $playlist->user ? $playlist->user->fullname : 'N/A',
                    'image' => $playlist->image ? $playlist->image : 'No Image',
                    'likes' => $playlist->likes,
                    'song_count' => $playlist->song_count,
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch playlists.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

    /**
     * Update the specified playlist.
     */
     
     
     
    public function update(Request $request)
    {

        $data = $request->validate([
            'id' => 'required|integer|exists:playlists,id',
            'playlist_name' => 'required|string|max:255',
            'user_id' => 'nullable|integer|exists:users,id', // Add user_id validation
            'image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            'playlisttype' => 'nullable|in:0,1',
        ]);
$data['playlisttype'] = $request->input('playlisttype', 0); // ✅ fallback to 0 if not set

        $playlist = Playlist::find($data['id']);

        if ($request->hasFile('image')) {
            GlobalFunction::deleteFile($playlist->image);
            $file = $request->file('image');
            $imagePath = GlobalFunction::saveFileAndGivePath($file, "playlist-images");
            $data['image'] = $imagePath;
        }

        $playlist = Playlist::find($data['id']);

$playlist->update($data);

        return response()->json(['success' => true, 'message' => 'Playlist updated successfully.']);
    }

    /**
     * Remove the specified playlist.
     */
    public function destroy(Request $request)
    {
        $id = $request->input('id');
        $playlist = Playlist::find($id);

        if (!$playlist) {
            return response()->json(['success' => false, 'message' => 'Playlist not found'], 404);
        }

        $playlist->delete();

        return response()->json(['success' => true, 'message' => 'Playlist deleted successfully.']);
    }
    
   public function getUserPlaylists($userId)
{
    try {
        // Fetch all playlists for the given user_id with song count
        $playlists = Playlist::where('user_id', $userId)
            ->with('user') // Include user details
            ->withCount('music') // ✅ Count number of songs in the playlist
            ->get();

        // Check if playlists exist
        if ($playlists->isEmpty()) {
            return response()->json([
                'status' => false,
                'message' => 'No playlists found for this user.',
                'data' => []
            ], 200);
        }

        // Return playlists data
        return response()->json([
            'status' => true,
            'message' => 'Playlists fetched successfully.',
            'data' => $playlists->map(function ($playlist) {
                return [
                    'id' => $playlist->id,
                    'playlist_name' => $playlist->playlist_name,
                    'image' => $playlist->image ? $playlist->image : 'No Image',
                    'user' => $playlist->user ? $playlist->user->fullname : 'N/A',
                    'likes' => $playlist->likes,
                    'song_count' => $playlist->music_count, // ✅ Correct way to count songs
                    'created_at' => $playlist->created_at,
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        // Handle exceptions
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch playlists.',
            'error' => $e->getMessage(),
        ], 500);
    }
}


public function getAllPlaylists()
{
    try {
        $playlists = Playlist::with('user') // Fetch user details
            ->withCount('music') // Count the number of songs in the playlist
            ->orderBy('likes', 'desc') // Sort by likes
            ->get();

        // Check if playlists exist
        if ($playlists->isEmpty()) {
            return response()->json([
                'status' => false,
                'message' => 'No playlists found.',
                'data' => [],
            ], 200);
        }

        // Return formatted response
        return response()->json([
            'status' => true,
            'message' => 'Playlists fetched successfully.',
            'data' => $playlists->map(function ($playlist) {
                return [
                    'id' => $playlist->id,
                    'playlist_name' => $playlist->playlist_name,
                    'image' => $playlist->image ? $playlist->image : 'No Image',
                    'user_id' => $playlist->user_id,
                    'playlisttype' => $playlist->playlisttype,
                    'user' => $playlist->user ? $playlist->user->fullname : 'N/A',
                    'likes' => $playlist->likes,
                    'created_at' => $playlist->created_at,
                    'song_count' => $playlist->music_count, // ✅ Song count added
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch playlists.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

public function getTopLikedPlaylists()
{
    try {
        // Fetch the top liked playlists in random order, limited to 20
        $playlists = Playlist::orderBy('likes', 'desc') // Order by likes in descending order
             ->having('music_count', '>', 0) 
            ->inRandomOrder() // Shuffle the results
            ->take(20) // Limit to 20 playlists
            ->withCount('music') // ✅ Add song count
            ->get();

        // Check if playlists exist
        if ($playlists->isEmpty()) {
            return response()->json([
                'status' => false,
                'message' => 'No playlists found.',
                'data' => []
            ], 200);
        }

        // Return playlists data
        return response()->json([
            'status' => true,
            'message' => 'Top liked playlists fetched successfully.',
            'data' => $playlists->map(function ($playlist) {
                return [
                    'id' => $playlist->id,
                    'title' => $playlist->playlist_name,
                    'image' => $playlist->image ? $playlist->image : 'No Image',
                    'likes' => $playlist->likes,
                    'song_count' => $playlist->music_count, // ✅ Add song count
                    'created_at' => $playlist->created_at,
                    'hometype' => 'playlist',
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        // Handle exceptions
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch playlists.',
            'error' => $e->getMessage(),
        ], 500);
    }
}


}


