<?php

namespace App\Http\Controllers;

use App\Models\Music;
use App\Models\Playlist;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Models\User;
use App\Models\GlobalFunction;
use App\Models\PlaylistMusic;

class PlaylistMusicController extends Controller
{
    public function createPlaylist(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'playlist_name' => 'required|string|max:255',
                'user_id' => 'required|exists:users,id',
                'image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            ]);

            $imagePath = $request->file('image') ? $request->file('image')->store('playlists') : null;

            $playlist = Playlist::create([
                'playlist_name' => $validatedData['playlist_name'],
                'user_id' => $validatedData['user_id'],
                'image' => $imagePath,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Playlist created successfully.',
                'data' => $playlist,
            ], 201);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to create playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addMusicToPlaylist(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'playlist_id' => 'required|exists:playlists,id',
                'music_id' => 'required|exists:music,id',
            ]);

            $playlist = Playlist::find($validatedData['playlist_id']);
            $playlist->music()->attach($validatedData['music_id']);

            return response()->json([
                'status' => true,
                'message' => 'Music added to playlist successfully.',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to add music to playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }


    public function playTrack(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'playlist_id' => 'required|exists:playlists,id',
                'music_id' => 'required|exists:music,id',
            ]);

            $playlist = Playlist::find($validatedData['playlist_id']);
            if (!$playlist->music()->where('id', $validatedData['music_id'])->exists()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Track not found in playlist.',
                ], 404);
            }

            $music = Music::find($validatedData['music_id']);
            $music->increment('plays');

            DB::table('music_play_logs')->insert([
                'music_id' => $music->id,
                'played_at' => now(),
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Track played successfully.',
                'data' => $music,
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to play track.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }


    public function deletePlaylist($id)
    {
        try {
            $playlist = Playlist::findOrFail($id);
            $playlist->delete();

            return response()->json([
                'status' => true,
                'message' => 'Playlist deleted successfully.',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to delete playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function likePlaylist($id)
    {
        try {
            $playlist = Playlist::findOrFail($id);
            $playlist->increment('likes');

            return response()->json([
                'status' => true,
                'message' => 'Playlist liked successfully.',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to like playlist.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function index(Request $request)
{
    if ($request->ajax()) {
        $query = PlaylistMusic::with(['playlist', 'music']);

        $totalData = $query->count(); // Total records count before filtering

        // Apply search filter if provided
        if ($search = $request->input('search.value')) {
            $query->whereHas('playlist', function ($q) use ($search) {
                $q->where('playlist_name', 'LIKE', "%{$search}%");
            })->orWhereHas('music', function ($q) use ($search) {
                $q->where('title', 'LIKE', "%{$search}%");
            });
        }

        $totalFiltered = $query->count(); // Total records count after filtering

        // Apply pagination and ordering
        $start = $request->input('start', 0);
        $length = $request->input('length', 10);
        $playlists = $query->offset($start)
            ->limit($length)
            ->get();

        // Build data for response
        $data = $playlists->map(function ($playlist) {
            return [
                'playlist_name' => $playlist->playlist->playlist_name ?? 'N/A',
                'music' => $playlist->music->title ?? 'N/A',
                'action' => '<button class="btn btn-success edit" data-id="' . $playlist->id . '">Edit</button>
                             <button class="btn btn-danger delete" data-id="' . $playlist->id . '">Delete</button>',
            ];
        });

        // Return JSON response
        return response()->json([
            "draw" => intval($request->input('draw')),
            "recordsTotal" => $totalData,
            "recordsFiltered" => $totalFiltered,
            "data" => $data,
        ]);
    }

    $all_playlists = Playlist::all();
    $all_music = Music::all();
    return view('playlist_music.index', compact('all_playlists', 'all_music'));
}

public function getPlaylistMusic(Request $request)
{
    $validatedData = $request->validate([
        'playlist_id' => 'required|exists:playlists,id', // Validate that the playlist_id exists in the playlists table
    ]);

    try {
        // Fetch the playlist with music details
        $playlist = Playlist::with(['music'])->where('id', $validatedData['playlist_id'])->first();

        if (!$playlist) {
            return response()->json([
                'status' => false,
                'message' => 'Playlist not found.',
            ], 404);
        }

        // Prepare the response data
        $data = [
            'playlist_id' => $playlist->id,
            'playlist_name' => $playlist->playlist_name,
            'user_id' => $playlist->user_id, // Include the user_id of the playlist owner
            'playlist_image' => $playlist->image ? GlobalFunction::createMediaUrl($playlist->image) : 'No Image',
            'music' => $playlist->music->map(function ($music) {
                return [
                    'id' => $music->id,
                    'category_id' => $music->category_id,
                    'lyrics' => $music ->lyrics,
                    'title' => $music->title,
                    'plays' => $music->plays,
                    'sound' => $music->sound ?$music->sound : 'No media',
                    'type' => $music ->type,
                    'is_featured' => $music->is_featured,
                    'image' => $music->image ? $music->image : 'No Image',
                    'music_url' => $music->sound ?$music->sound : 'No media',
                    'likes' => $music->likes,
                    'active' => $music->active,
                    'created_at' => $music->created_at,
                    'updated_at' => $music->updated_at,
                ];
            }),
        ];

        return response()->json([
            'status' => true,
            'message' => 'Playlist music fetched successfully.',
            'data' => $data,
        ], 200);
    } catch (\Exception $e) {
        // Handle any exceptions
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch playlist music.',
            'error' => $e->getMessage(),
        ], 500);
    }
}


    /**
     * Store a newly created playlist.
     */
    public function store(Request $request)
    {
        $data = $request->validate([
            'playlist_id' => 'required|integer',
            'music_id' => 'required|integer', 
        ]);

        PlaylistMusic::create($data);

        return response()->json(['success' => true, 'message' => 'Playlist Music added successfully.']);
    }

    /**
     * Show the form for editing the specified playlist.
     */
    public function edit($id)
    {
        $playlist = PlaylistMusic::with('music', 'playlist')->find($id);

        if (!$playlist) {
            return response()->json(['success' => false, 'message' => 'Playlist Music not found'], 404);
        }

        return response()->json([
            'success' => true,
            'playlist_id' => $playlist->playlist->id,
            'music_id' => $playlist->music->id,
        ]);
    }

    /**
     * Update the specified playlist.
     */
    public function update(Request $request)
    {
        $data = $request->validate([
            'id' => 'required|integer|exists:playlist_music,id',
            'playlist_id' => 'required|integer|exists:playlists,id',
            'music_id' => 'required|integer|exists:music,id', 
        ]);


        $playlist = PlaylistMusic::find($data['id']);


        $playlist->update($data);

        return response()->json(['success' => true, 'message' => 'Playlist updated successfully.']);
    }

    /**
     * Remove the specified playlist.
     */
    public function destroy(Request $request)
    {
        $id = $request->input('id');
        $playlist = PlaylistMusic::find($id);

        if (!$playlist) {
            return response()->json(['success' => false, 'message' => 'Playlist Music not found'], 404);
        }

        $playlist->delete();

        return response()->json(['success' => true, 'message' => 'Playlist Music deleted successfully.']);
    }
    
    
    public function removeMusicFromPlaylist(Request $request)
{
    try {
        $validatedData = $request->validate([
            'playlist_id' => 'required|exists:playlists,id',
            'music_id' => 'required|exists:music,id',
        ]);

        $playlist = Playlist::find($validatedData['playlist_id']);

        // Check if the music exists in the playlist
        if (!$playlist->music()->where('music_id', $validatedData['music_id'])->exists()) {
            return response()->json([
                'status' => false,
                'message' => 'Music not found in the playlist.',
            ], 404);
        }

        // Detach the music from the playlist
        $playlist->music()->detach($validatedData['music_id']);

        return response()->json([
            'status' => true,
            'message' => 'Music removed from playlist successfully.',
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to remove music from playlist.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

}
