<?php

namespace App\Http\Controllers;
use Illuminate\Support\Str;

use App\Models\GlobalFunction;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use App\Models\PodcastCategory;
use App\Models\Artist;

class PodcastCategoryController extends Controller
{
    public function index()
    {
        $podcastCategories = PodcastCategory::with('artist')->get();
        $podcastCategories->map(function ($category) {
            $category->image_url = $category->image ? GlobalFunction::createMediaUrl($category->image) : null;
            return $category;
        });

        $artists = Artist::all(); // Fetch all artists for dropdown
        return view('podcast_categories.index', compact('podcastCategories', 'artists'));
    }

    public function podcastCategoryList(Request $request)
    {
        $totalData = PodcastCategory::count();
        $rows = PodcastCategory::with('artist')->orderBy('id', 'DESC')->get();

        $data = [];
        foreach ($rows as $item) {
            $edit = '<a href="#" data-category_name="' . $item->name . '" 
                     data-description="' . $item->description . '" 
                     data-artist_id="' . $item->artist_id . '" 
                     data-tags="' . $item->tags . '" 
                     data-image_url="' . ($item->image ? GlobalFunction::createMediaUrl($item->image) : '') . '" 
                     rel="' . $item->id . '" 
                     class="btn edit btn-success me-3">Edit</a>';

            $delete = '<a href="#" class="btn delete btn-danger" rel="' . $item->id . '">Delete</a>';
            $action = '<span class="float-end">' . $edit . $delete . '</span>';

            $data[] = [
                $item->name,
               Str::limit($item->description, 50, '...'),
                $item->artist ? $item->artist->name : 'N/A',
                $item->tags,
                $item->image ? '<img src="' . GlobalFunction::createMediaUrl($item->image) . '" alt="Image" width="50" height="50">' : 'N/A',
                $action
            ];
        }

        $json_data = [
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($totalData),
            'recordsFiltered' => $totalData,
            'data' => $data,
        ];

        return response()->json($json_data);
    }

    public function addPodcastCategory(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'description' => 'nullable|string|max:1000',
            'artist_id' => 'nullable|exists:artists,id',
            'tags' => 'nullable|string',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        $podcastCategory = new PodcastCategory();
        $podcastCategory->name = $request->name;
        $podcastCategory->description = $request->description;
        $podcastCategory->artist_id = $request->artist_id;
        $podcastCategory->tags = $request->tags;

        if ($request->hasFile('image')) {
            $podcastCategory->image = GlobalFunction::saveFileAndGivePath($request->file('image'), "podcast_categories");
        }

        $podcastCategory->save();

        return response()->json([
            'status' => true,
            'message' => 'Podcast Category Added Successfully',
            'data' => $podcastCategory,
        ]);
    }

    public function deletePodcastCategory(Request $request)
    {
        $podcastCategory = PodcastCategory::find($request->id);

        if ($podcastCategory) {
            if ($podcastCategory->image) {
                GlobalFunction::deleteFile($podcastCategory->image); // Delete image from S3
            }
            $podcastCategory->delete();
            return response()->json([
                'status' => true,
                'message' => 'Podcast Category Deleted Successfully',
            ]);
        }

        return response()->json([
            'status' => false,
            'message' => 'Podcast Category Not Found',
        ]);
    }

    public function updatePodcastCategory(Request $request)
    {
        $request->validate([
            'name' => 'required|string|max:255',
            'description' => 'nullable|string|max:1000',
            'artist_id' => 'nullable|exists:artists,id',
            'tags' => 'nullable|string',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        $podcastCategory = PodcastCategory::find($request->id);

        if (!$podcastCategory) {
            return response()->json([
                'status' => false,
                'message' => 'Podcast Category Not Found',
            ]);
        }

        $podcastCategory->name = $request->name;
        $podcastCategory->description = $request->description;
        $podcastCategory->artist_id = $request->artist_id;
        $podcastCategory->tags = $request->tags;

        if ($request->hasFile('image')) {
            if ($podcastCategory->image) {
                GlobalFunction::deleteFile($podcastCategory->image); // Delete old image
            }
            $podcastCategory->image = GlobalFunction::saveFileAndGivePath($request->file('image'), "podcast_categories");
        }

        $podcastCategory->save();

        return response()->json([
            'status' => true,
            'message' => 'Podcast Category Updated Successfully',
            'data' => $podcastCategory,
        ]);
    }

    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'name' => 'required|string|max:255',
                'description' => 'nullable|string|max:1000',
                'artist_id' => 'nullable|exists:artists,id',
                'tags' => 'nullable|string',
                'image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            ]);

            if ($request->hasFile('image')) {
                $validatedData['image'] = GlobalFunction::saveFileAndGivePath($request->file('image'), "podcast_categories");
            }

            $podcastCategory = PodcastCategory::create($validatedData);

            return response()->json([
                'message' => 'Podcast Category created successfully',
                'data' => $podcastCategory
            ], 200);
        } catch (\Exception $e) {
            Log::error('Error creating PodcastCategory: ' . $e->getMessage());

            return response()->json([
                'message' => 'Failed to create Podcast Category',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function index_api()
    {
        try {
            $podcastCategories = PodcastCategory::with('artist')->get();
            $podcastCategories->map(function ($category) {
                $category->image_url = $category->image ? GlobalFunction::createMediaUrl($category->image) : null;
                return $category;
            });

            return response()->json([
                'message' => 'Podcast Categories retrieved successfully',
                'data' => $podcastCategories
            ], 200);
        } catch (\Exception $e) {
            Log::error('Error fetching PodcastCategories: ' . $e->getMessage());

            return response()->json([
                'message' => 'Failed to retrieve Podcast Categories',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function show_api($id)
    {
        try {
            $podcastCategory = PodcastCategory::with('artist')->find($id);

            if (!$podcastCategory) {
                return response()->json([
                    'message' => 'Podcast Category not found'
                ], 404);
            }

            $podcastCategory->image_url = $podcastCategory->image ? GlobalFunction::createMediaUrl($podcastCategory->image) : null;

            return response()->json([
                'message' => 'Podcast Category retrieved successfully',
                'data' => $podcastCategory
            ], 200);
        } catch (\Exception $e) {
            Log::error('Error fetching PodcastCategory: ' . $e->getMessage());

            return response()->json([
                'message' => 'Failed to retrieve Podcast Category',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    

    public function getPodcastsWithEpisodes()
    {
        $podcasts = PodcastCategory::with([
            'podcasts' => function ($query) {
                $query->select(
                    'id', 
                    'title', 
                    'description', 
                    'podcast_category_id', 
                    'episode', 
                    'episode_number', 
                    'duration', 
                    'likes', 
                    'plays', 
                    'type',
                    'start_date', 
                    'end_date'
                )->orderBy('created_at', 'desc');
            }
        ])->get();

        if ($podcasts->isEmpty()) {
            return response()->json([
                'status' => false,
                'message' => 'No podcasts found',
                'data' => []
            ], 200);
        }

        return response()->json([
            'status' => true,
            'message' => 'Podcasts retrieved successfully',
            'data' => $podcasts
        ], 200);
    }


}
