<?php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\GuidedSessionTemplate;
use App\Models\ActivityBlueprint;
use App\Models\Music;
use App\Models\Podcast;
use App\Models\VibeCheck;
use App\Models\UserSessionLog;

class DynamicSessionController extends Controller
{
    public function launch(Request $request, $id)
    {
        $userId = $request->input('user_id');
        $template = GuidedSessionTemplate::findOrFail($id);

        // Step 1: Load user vibe + time tag
        $vibe = VibeCheck::where('user_id', $userId)->latest()->first()?->vibe_type ?? null;
        $hour = now()->hour;
        $timeTag = $hour < 12 ? 'morning' : ($hour < 18 ? 'afternoon' : 'night');

        // Step 2: Merge tags
        $allTags = array_filter(array_unique(array_merge(
            $template->goal_tags ?? [],
            $template->mood_tags ?? [],
            $template->sound_tags ?? [],
            $template->cannabis_tags ?? [],
            $template->use_case_tags ?? [],
            [$template->time_of_day, $vibe, $timeTag]
        )));

        // Step 3: Pull blueprints
        $blueprints = ActivityBlueprint::whereIn('trigger_tag', $allTags)
            ->where('active', true)
            ->take(3)
            ->get();

        // Step 4: Load tag config from activity_tags.php
        $config = config('activity_tags');
        $tagRules = $config['tag_rules'];
        $defaultCounts = $config['default'];

        // Step 5: Generate activities
        $activities = $blueprints->map(function ($bp) use ($allTags, $tagRules, $defaultCounts) {
            $tags = array_map('strtolower', $bp->default_tags ?? []);

            // Default content count
            $musicCount = $defaultCounts['music'];
            $podcastCount = $defaultCounts['podcasts'];

            // Apply rules from tag config
            foreach ($tags as $tag) {
                if (isset($tagRules[$tag])) {
                    $musicCount = $tagRules[$tag]['music'];
                    $podcastCount = $tagRules[$tag]['podcasts'];
                    break;
                }
            }

            $music = Music::where('active', 1)->where(function ($q) use ($allTags) {
                foreach ($allTags as $tag) {
                    $q->orWhere('tag', 'like', "%$tag%");
                }
            })->limit($musicCount)->get();

            $podcasts = Podcast::where('active', 1)->where(function ($q) use ($allTags) {
                foreach ($allTags as $tag) {
                    $q->orWhere('description', 'like', "%$tag%");
                }
            })->limit($podcastCount)->get();

            return [
                'title' => $bp->title,
                'description' => $bp->description,
                'duration' => $bp->default_duration,
                'tags_used' => $bp->default_tags,
                'eq_profile' => $bp->eq_profile,
                'music' => $music,
                'podcasts' => $podcasts
            ];
        });

        // Step 6: Log session
        $log = UserSessionLog::create([
            'user_id' => $userId,
            'guided_session_template_id' => $template->id,
            'vibe_type' => $vibe,
            'time_of_day' => $timeTag,
            'tags_used' => $allTags,
            'total_duration' => $activities->sum('duration'),
            'launched_at' => now()
        ]);

        return response()->json([
            'status' => true,
            'session' => [
                'session_log_id' => $log->id,
                'title' => $template->title,
                'description' => $template->description,
                'vibe_used' => $vibe,
                'time_tag' => $timeTag,
                'tags_combined' => $allTags,
                'total_duration' => $activities->sum('duration'),
                'activities' => $activities
            ]
        ]);
    }
}
