<?php

namespace App\Http\Controllers;

use App\Models\EqComment;
use App\Models\EqCommentLike;

use Exception;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use App\Models\EqStrain;
use App\Models\User;

class EqCommentController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
public function index(Request $request, $strain_id)
{
    try {
        // ✅ Fetch comments with pagination
        $comments = EqComment::where('strain_id', $strain_id)
            ->with(['user', 'strain', 'likes'])
            ->orderBy('created_at', 'desc')
            ->paginate(10); // ✅ Pagination Applied

        // ✅ Format the response
        $formattedComments = $comments->map(function ($comment) {
            return [
                'id' => $comment->id,
                'user_id' => $comment->user->id,
                'strain_id' => $comment->strain->id,
                'user' => [
                    'id' => $comment->user->id,
                    "identity" => $comment->user->identity,
                    'fullname' => $comment->user->fullname,
                    "image" => $comment->user->image,
                ],
                'strain' => [
                    'id' => $comment->strain->id,
                    'name' => $comment->strain->strain, // ✅ Ensure correct field name
                ],
                'comment' => $comment->comment,
                'likes' => $comment->likes ? count($comment->likes) : 0, // ✅ Count likes
                'created_at' => $comment->created_at,
            ];
        });

        // ✅ Include pagination metadata
        return response()->json([
            'status' => true,
            'message' => 'Comments fetched successfully.',
            'data' => [
                'comments' => $formattedComments,
                'pagination' => [
                    'total' => $comments->total(),
                    'current_page' => $comments->currentPage(),
                    'last_page' => $comments->lastPage(),
                    'per_page' => $comments->perPage(),
                    'next_page_url' => $comments->nextPageUrl(),
                    'prev_page_url' => $comments->previousPageUrl(),
                ],
            ],
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch comments.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    //  ALTER TABLE `eq_comments` ADD `likes` INT NULL AFTER `comment`;

    public function store(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'user_id' => 'required|exists:users,id',
                'strain_id' => 'required|exists:eq_strain,id',
                'comment' => 'required|string|max:500',
            ]);

            $comment = EqComment::create([
                'user_id' => $validatedData['user_id'],
                'strain_id' => $validatedData['strain_id'],
                'comment' => $validatedData['comment'],
            ]);

            $comment->load(['user', 'strain']);

            return response()->json([
                'message' => 'Comment posted successfully',
                'data' => $comment
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Failed to post comment',
                'error' => $e->getMessage()
            ], 500);
        }
    }

  public function like(Request $request, $comment_id)
{
    try {
        $user_id = $request->user_id;

        $comment = EqComment::findOrFail($comment_id);

        // Check if the user has already liked this comment
        if ($comment->likedByUsers()->where('user_id', $user_id)->exists()) {
            return response()->json([
                'message' => 'You have already liked this comment',
                'data' => $comment
            ], 400);
        }

        // Add the like
        $comment->likedByUsers()->attach($user_id);
        $comment->increment('likes');

        return response()->json([
            'message' => 'Comment liked successfully',
            'data' => $comment
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to like the comment',
            'error' => $e->getMessage()
        ], 500);
    }
}

public function unlike(Request $request, $comment_id)
{
    try {
        $user_id = $request->user_id;

        $comment = EqComment::findOrFail($comment_id);

        // Check if the user has liked this comment before unliking
        if (!$comment->likedByUsers()->where('user_id', $user_id)->exists()) {
            return response()->json([
                'message' => 'You have not liked this comment yet',
                'data' => $comment
            ], 400);
        }

        // Remove the like record
        $comment->likedByUsers()->detach($user_id);
        $comment->decrement('likes');

        return response()->json([
            'message' => 'Comment unliked successfully',
            'data' => $comment
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to unlike the comment',
            'error' => $e->getMessage()
        ], 500);
    }
}


    public function update(Request $request, $comment_id)
    {
        try {
            $validatedData = $request->validate([
                'comment' => 'required|string|max:500',
            ]);

            $comment = EqComment::findOrFail($comment_id);

            if ($request->user()->id != $comment->user_id) {
                return response()->json([
                    'message' => 'Unauthorized action'
                ], 403);
            }

            $comment->update([
                'comment' => $validatedData['comment'],
            ]);

            $comment->load(['user', 'strain']);

            return response()->json([
                'message' => 'Comment updated successfully',
                'data' => $comment
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Failed to update comment',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function indexviewnew()
    {
        $users = User::all();
        $strains = EqStrain::all();
        return view('eq_comments.index', compact('users','strains'));
    }

    public function indexnew(Request $request)
    {
        $query = EqComment::with(['user', 'strain']);
        $totalData = $query->count();

        if ($search = $request->input('search.value')) {
            $query->where('comment', 'LIKE', "%{$search}%")
                  ->orWhereHas('user', fn($q) => $q->where('fullname', 'LIKE', "%{$search}%"));
        }

        $comments = $query->offset($request->input('start'))
                          ->limit($request->input('length'))
                          ->get();

        return response()->json([
            "draw" => intval($request->input('draw')),
            "recordsTotal" => $totalData,
            "recordsFiltered" => $totalData,
            "data" => $comments->map(function ($comment) {
                return [
                    'user' => $comment->user->fullname,
                    'strain' => $comment->strain->strain ? : 'N/A',
                    'comment' => $comment->comment,
                    'likes' => $comment->likes,
                    'action' => '<button class="btn btn-success edit" data-id="' . $comment->id . '">Edit</button>
                                 <button class="btn btn-danger delete" data-id="' . $comment->id . '">Delete</button>'
                ];
            }),
        ]);
    }
    public function editnew($id)
{
    // Fetch the comment by its ID
    $comment = EqComment::with('user', 'strain')->find($id);

    // Check if the comment exists
    if (!$comment) {
        return response()->json([
            'status' => false,
            'message' => 'Comment not found'
        ], 404);
    }

    // Return the comment as a JSON response
    return response()->json([
        'status' => true,
        'comment' => $comment
    ]);
}


    public function storenew(Request $request)
    {
        $data = $request->validate([
            'user_id' => 'required',
            'comment' => 'required',
            'strain_id' => 'required',
        ]);

        EqComment::create($data);

        return response()->json(['status' => true]);
    }

    public function updatenew(Request $request)
    {
        $comment = EqComment::find($request->id);
        if (!$comment) {
            return response()->json(['status' => false, 'message' => 'Comment not found']);
        }

        $data = $request->validate([
            'user_id' => 'required|exists:users,id',  // Make sure user exists
            'strain_id' => 'required',  // Make sure strain exists
            'comment' => 'required|string',
        ]);

        $comment->update($data);

        return response()->json(['status' => true, 'message' => 'Comment updated successfully']);
    }

    public function destroynew(Request $request)
    {
        EqComment::find($request->id)->delete();
        return response()->json(['status' => true]);
    }
    
  public function toggleLike(Request $request, $comment_id)
{
    try {
        $validated = $request->validate([
            'user_id' => 'required|exists:users,id',
        ]);

        $user_id = $validated['user_id'];

        // Check if the user has already liked the comment
        $existingLike = EqCommentLike::where('user_id', $user_id)
            ->where('comment_id', $comment_id)
            ->first();

        if ($existingLike) {
            // Unlike: Remove the like
            $existingLike->delete();
            return response()->json([
                'message' => 'Comment unliked successfully',
                'is_liked' => false,
            ], 200);
        } else {
            // Like: Add new record
            EqCommentLike::create([
                'user_id' => $user_id,
                'comment_id' => $comment_id,
            ]);

            return response()->json([
                'message' => 'Comment liked successfully',
                'is_liked' => true,
            ], 200);
        }
    } catch (\Exception $e) {
        return response()->json([
            'message' => 'Failed to toggle like',
            'error' => $e->getMessage(),
        ], 500);
    }
}


    
}
