<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\GuidedSession;
use App\Models\SessionActivity;
use App\Models\Music;
use App\Models\Podcast;
use App\Models\VibeCheck;


class GuidedSessionController extends Controller
{
public function getRecommendedSession(Request $request)
{
    $userId = $request->input('user_id');
    $vibe = VibeCheck::where('user_id', $userId)->latest()->first();
    $timeOfDay = now()->format('H'); // Get current hour

    // Determine time-based tag
    $timeTag = match (true) {
        $timeOfDay < 12 => 'morning',
        $timeOfDay < 18 => 'afternoon',
        default => 'night',
    };

    // Pull top strain or default
    $topStrain = DB::table('eq_strain')
        ->select('type')
        ->orderBy('likes', 'desc')
        ->first()?->type ?? 'hybrid';

    // Load a session template
    $session = GuidedSession::with('activities')->inRandomOrder()->first();
    if (!$session) {
        return response()->json(['status' => false, 'message' => 'No session available']);
    }

    // Build full session with activities + matched content
    $activities = $session->activities->map(function ($activity) use ($vibe, $topStrain, $timeTag) {
        $tags = array_merge(
            json_decode($activity->tag_filters ?? '[]'),
            [$vibe->vibe_type, strtolower($topStrain), $timeTag]
        );

        // Match music
        $music = Music::where('active', 1)
            ->where(function ($q) use ($tags) {
                foreach ($tags as $tag) {
                    $q->orWhere('tag', 'like', "%$tag%");
                }
            })
            ->limit(1)
            ->get();

        // Match podcast
        $podcast = Podcast::where('active', 1)
            ->where(function ($q) use ($tags) {
                foreach ($tags as $tag) {
                    $q->orWhere('description', 'like', "%$tag%");
                }
            })
            ->limit(1)
            ->get();

        return [
            'title' => $activity->title,
            'description' => $activity->description,
            'duration' => $activity->duration,
            'eq_profile' => $activity->eq_profile,
            'tags_used' => $tags,
            'music' => $music,
            'podcast' => $podcast,
        ];
    });

    return response()->json([
        'status' => true,
        'session' => [
            'title' => $session->title,
            'description' => $session->description,
            'total_duration' => $activities->sum('duration'),
            'activities' => $activities,
        ]
    ]);
}


public function listAllSessions(Request $request)
{
    $sessions = GuidedSession::with(['activities'])->where('active', true)->get();

    $response = $sessions->map(function ($session) {
        return [
            'id' => $session->id,
            'title' => $session->title,
            'description' => $session->description,
            'estimated_duration' => $session->activities->sum('duration'),
            'activity_count' => $session->activities->count(),
            'vibe_preview' => $session->activities->pluck('tag_filters')->flatten()->unique()->take(3),
        ];
    });

    return response()->json([
        'status' => true,
        'data' => $response
    ]);
}
public function show($id)
{
    $session = GuidedSession::with([
        'activities.musicTracks',
        'activities.podcasts'
    ])->find($id);

    if (!$session) {
        return response()->json([
            'status' => false,
            'message' => 'Session not found'
        ], 404);
    }

    $activities = $session->activities->map(function ($activity) {
        return [
            'title' => $activity->title,
            'description' => $activity->description,
            'duration' => $activity->duration,
            'eq_profile' => $activity->eq_profile,
            'music' => $activity->musicTracks->map(function ($music) {
                return [
                    'id' => $music->id,
                    'title' => $music->title,
                    'tag' => $music->tag,
                    'sound' => $music->sound,
                ];
            }),
            'podcasts' => $activity->podcasts->map(function ($podcast) {
                return [
                    'id' => $podcast->id,
                    'title' => $podcast->title,
                    'duration' => $podcast->duration,
                    'episode' => $podcast->episode,
                ];
            }),
        ];
    });

    return response()->json([
        'status' => true,
        'data' => [
            'id' => $session->id,
            'title' => $session->title,
            'description' => $session->description,
            'total_duration' => $activities->sum('duration'),
            'activities' => $activities
        ]
    ]);
}
}
