<?php

namespace App\Http\Controllers;

use App\Models\Album;
use App\Models\GlobalFunction;
use App\Models\Music;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class MusicController extends Controller
{
    public function musics()
    {
        // 1) Retrieve albums (with their artists if you want to display artist name in the dropdown)
        $categories = Album::with('artist')->get();

        // 2) Retrieve all distinct tags from the Music table (simple approach)
        //    If your 'tag' column contains comma-separated tags, you may need a more advanced approach.
        $tags = Music::whereNotNull('tag')
            ->where('tag', '!=', '')
            ->distinct()
            ->pluck('tag'); // returns a collection of tag strings
        // This might return "indica", "sativa", "hybrid", or even "indica, sativa" if stored that way.

        // 3) Retrieve artist names from the albums (assuming each album has an artist relationship)
        //    If you have a dedicated Artist model/table, you can query that directly.
        $artists = DB::table('artists')->pluck('name', 'id');
        // Or you might do:
        // $artists = Artist::all(); // if you have an Artist model

        return view('musics', [
            'categories' => $categories,
            'tags' => $tags,
            'artists' => $artists,
        ]);
    }

  public function musicList(Request $request)
{
    try {
        // Initialize columns and retrieve input parameters
        $columns = [
            0 => 'id',
            1 => 'image',
        ];
        $limit = $request->input('length', 10); // Default to 10 records per page
        $start = $request->input('start', 0);
        $order = $columns[$request->input('order.0.column')] ?? 'id'; // Default to 'id'
        $dir   = $request->input('order.0.dir', 'DESC');              // Default to descending order
        $search = $request->input('search.value');

        // Filters
        $filterAlbum  = $request->input('album');  
        $filterType   = $request->input('type');    
        $filterTag    = $request->input('tag');     
        $filterArtist = $request->input('artist');  

        // Query for total records
        $totalData = Music::count();

        // Build base query
        $query = Music::query();

        // Apply global search (by title) if search term exists
        if (!empty($search)) {
            $query->where('title', 'LIKE', "%{$search}%");
        }

        // Apply filters
        if (!empty($filterAlbum)) {
            $query->where('category_id', $filterAlbum);
        }

        if ($filterType !== null && $filterType !== '') {
            $query->where('type', $filterType);
        }

        if (!empty($filterTag)) {
            $query->where('tag', 'LIKE', "%{$filterTag}%");
        }

        if (!empty($filterArtist)) {
            $query->whereHas('category.artist', function ($q) use ($filterArtist) {
                $q->where('name', 'LIKE', "%{$filterArtist}%");
            });
        }

        $totalFiltered = $query->count(); // Recalculate total for filtered records

        // Retrieve paginated and ordered results
        $result = $query->offset($start)
                        ->limit($limit)
                        ->orderBy($order, $dir)
                        ->get();

        // Prepare response data
        $data = [];
        foreach ($result as $item) {
            $imageUrl = $item->image ? GlobalFunction::createMediaUrl($item->image) : './asset/img/placeholder-image.png';
            $musicUrl = $item->sound ? GlobalFunction::createMediaUrl($item->sound) : './asset/audio/placeholder.mp3';

            $image = "<img src='{$imageUrl}' class='tbl_img lazy'>";
            $type = $item->type == 1
                ? '<span class="badge theme-bg px-3">Premium</span>'
                : '<span class="badge badge-info px-3">Free</span>';
            $plays = "<span class='badge badge-warning px-3 text-dark fw-semibold plays-count'>{$item->plays}</span>";

            $featured = $this->getCheckboxSlider($item->id, $item->is_featured, 'featured');
            $active   = $this->getCheckboxSlider($item->id, $item->active, 'active');

            $music = "<audio id='show_music' src='{$musicUrl}' controls style='width:200px;'></audio>";
            $title = "<div class='itemDescription'>{$item->title}</div>";
            $tags  = "<div class='itemDescription'>{$item->tag}</div>";

            // ✅ Lyrics are removed from table data, only included in edit button attributes
            $edit = "<a href='#'
                        rel='{$item->id}'
                        data-title='{$item->title}'
                        data-lyrics='" . htmlspecialchars($item->lyrics, ENT_QUOTES, 'UTF-8') . "'                        data-tag='{$item->tag}'
                        data-image='{$imageUrl}'
                        data-category_id='{$item->category_id}'
                        data-type='{$item->type}'
                        data-sound='{$musicUrl}'
                        class='btn edit btn-success me-3'>Edit</a>";

            $delete = "<a href='#' class='btn delete btn-danger' rel='{$item->id}'>Delete</a>";

            $action = "<span class='float-end'>{$edit}{$delete}</span>";
            $artist = $item->category && $item->category->artist
                ? $item->category->artist->name
                : "No Artist Found";

            $data[] = [
                $image,
                $music,
                $title,
                $tags,
                $plays,
                $item->category->title ?? 'No Category',
                $type,
                $featured,
                $active,
                $artist,
                $action,
            ];
        }

        // Return the response as JSON
        return response()->json([
            'draw'            => intval($request->input('draw', 1)),
            'recordsTotal'    => $totalData,
            'recordsFiltered' => $totalFiltered,
            'data'            => $data,
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status'  => false,
            'message' => 'Failed to fetch music list.',
            'error'   => $e->getMessage(),
        ], 500);
    }
}


    /**
     * Helper function to generate a checkbox slider for featured and active statuses.
     */
    private function getCheckboxSlider($id, $status, $type)
    {
        $checked = $status ? 'checked' : '';
        return <<<HTML
        <div class="checkbox-slider">
            <label>
                <input type="checkbox" class="d-none {$type}" {$checked} rel="{$id}" value="{$status}">
                <span class="toggle_background">
                    <div class="circle-icon"></div>
                    <div class="vertical_line"></div>
                </span>
            </label>
        </div>
        HTML;
    }
    public function addMusic(Request $request)
    {
        $music = new Music();
        $music->title = $request->title;
        $music->category_id = $request->category_id;
        $music->type = $request->type;
        $music->lyrics = $request->lyrics;
        $music->tag = $request->tag;

        $imageFile = $request->file('image');
        $imagePath = GlobalFunction::saveFileAndGivePath($imageFile, "music-images");
        $music->image = $imagePath;

        $audioFile = $request->file('audio');
        $audioPath = GlobalFunction::saveFileAndGivePath($audioFile, "music-sounds");
        $music->sound = $audioPath;

        $music->save();

        return response()->json([
            'status' => true,
            'message' => 'Music added successfully',
            'data' => $music,
        ]);
    }

    public function updateFeatured(Request $request)
    {
        $music = Music::where('id', $request->id)->first();
        if ($music == null) {
            return response()->json([
                'status' => false,
                'message' => 'Music not found',
            ]);
        }

        $music->is_featured = $request->is_featured;
        $music->save();

        return response()->json([
            'status' => true,
            'message' => 'Music updated successfully',
        ]);
    }

    public function updateMusicActive(Request $request)
    {
        $music = Music::where('id', $request->id)->first();
        if ($music == null) {
            return response()->json([
                'status' => false,
                'message' => 'Music not found',
            ]);
        }

        $music->active = $request->active;
        $music->save();

        return response()->json([
            'status' => true,
            'message' => 'Music updated successfully',
        ]);
    }


    public function updateMusic(Request $request)
    {
        $music = Music::where('id', $request->music_id)->first();

        if ($music == null) {
            return response()->json([
                'status' => false,
                'message' => 'Music Not Found',
            ]);
        } else {

            if ($request->has('title')) {
                $music->title = $request->title;
            }
            if ($request->has('category_id')) {
                $music->category_id = $request->category_id;
            }
            if ($request->has('type')) {
                $music->type = $request->type;
            }

            if ($request->hasFile('image')) {
                GlobalFunction::deleteFile($music->image);
                $file = $request->file('image');
                $imagePath = GlobalFunction::saveFileAndGivePath($file, "music-images");
                $music->image = $imagePath;
            }

            if ($request->hasFile('audio')) {
                GlobalFunction::deleteFile($music->sound);
                $audioFile = $request->file('audio');
                $filePath = GlobalFunction::saveFileAndGivePath($audioFile, "music-sounds");
                $music->sound = $filePath;
            }

            $music->lyrics = $request->lyrics;
            $music->tag = $request->tag;

            $music->save();

            return response()->json([
                'status' => true,
                'message' => 'Music updated successfully',
            ]);
        }
    }

public function topPlayedCategories()
{
    try {
        // Aggregate plays across categories and order by total play count
        $topCategories = DB::table('music')
            ->select('category_id', DB::raw('SUM(plays) as total_plays'))
            ->groupBy('category_id')
            ->orderBy('total_plays', 'desc')
            ->take(10) // Limit to top 10 categories
            ->get();

        if ($topCategories->isEmpty()) {
            return response()->json([
                'status' => true,
                'message' => 'No categories found with plays.',
                'data' => []
            ], 200);
        }

        // Get category details
        $categoryIds = $topCategories->pluck('category_id');
        $categories = Album::whereIn('id', $categoryIds)
            ->orderByRaw("FIELD(id, {$categoryIds->implode(',')})")
            ->get();

        // Combine category details with their total plays
        $categoriesWithPlays = $categories->map(function ($category) use ($topCategories) {
            $totalPlays = $topCategories->firstWhere('category_id', $category->id)->total_plays ?? 0;
            return [
                'id' => $category->id,
                'name' => $category->title,
                'total_plays' => $totalPlays,
            ];
        });

        return response()->json([
            'status' => true,
            'message' => 'Top played categories fetched successfully.',
            'data' => $categoriesWithPlays
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch top played categories.',
            'error' => $e->getMessage()
        ], 500);
    }
}

    public function deleteMusic(Request $request)
    {
        $music = Music::where('id', $request->music_id)->first();

        if ($music == null) {
            return response()->json([
                'status' => false,
                'message' => 'Music Not Found',
            ]);
        }
        GlobalFunction::deleteFile($music->image);
        GlobalFunction::deleteFile($music->sound);
        $music->delete();

        return response()->json([
            'status' => true,
            'message' => 'Music delete successfully',
        ]);
    }

   public function playSound(Request $request)
{
    try {
        // Validate the request
        $validatedData = $request->validate([
            'music_id' => 'required|exists:music,id',
            'user_id' => 'required|exists:users,id',
        ]);

        // Find the music
        $music = Music::find($validatedData['music_id']);

        if (!$music) {
            return response()->json([
                'status' => false,
                'message' => 'Music not found.',
            ], 404);
        }

        // Increment the play count
        $music->increment('plays');

        // Log the play with music_id and user_id
        DB::table('music_play_logs')->insert([
            'music_id' => $music->id,
            'user_id' => $validatedData['user_id'], // Log the user ID
            'played_at' => now(),
        ]);

        return response()->json([
            'status' => true,
            'message' => 'Music played successfully.',
            'data' => $music,
        ], 200);
    } catch (\Exception $e) {
        // Handle exceptions
        return response()->json([
            'status' => false,
            'message' => 'Failed to play the music.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

    public function likeSound(Request $request)
    {
        $music = Music::find($request->music_id);
        
        if (!$music) {
            return response()->json([
                'status' => false,
                'message' => 'Music not found.',
            ], 404);
        }

        $music->likes += 1;
        $music->save();

        return response()->json([
            'status' => true,
            'message' => 'Music liked successfully.',
            'data' => $music,
        ], 200);
    }




 public function trendingMusic()
{
    try {
        $startOfWeek = now()->subDays(7)->startOfDay(); // 7 days ago
        $endOfToday = now()->endOfDay(); // now

        $trendingMusic = DB::table('music_play_logs')
            ->select('music_id', DB::raw('COUNT(*) as play_count'))
            ->whereBetween('played_at', [$startOfWeek, $endOfToday])
            ->groupBy('music_id')
            ->orderBy('play_count', 'desc')
            ->take(10)
            ->get();

        $musicIds = $trendingMusic->pluck('music_id');

        if ($musicIds->isEmpty()) {
            return response()->json([
                'status' => true,
                'message' => 'No trending music found this week.',
                'data' => [],
            ], 200);
        }

        $musicDetails = Music::whereIn('id', $musicIds)
            ->orderByRaw("FIELD(id, {$musicIds->implode(',')})")
            ->get();

        return response()->json([
            'status' => true,
            'message' => 'Trending music this week fetched successfully.',
            'data' => $musicDetails->map(function ($music) {
                return [
                    'id' => $music->id,
                    'title' => $music->title,
                    'category_id' => $music->category_id ?? 'No Category',
                    'plays' => $music->plays,
                    'likes' => $music->likes,
                    'tag' => $music->tag,
                    'lyrics' => $music->lyrics,
                    'active' => $music->active,
                    'sound' => $music->sound,
                    'image' => $music->image ? $music->image : 'No Image',
                    'created_at' => $music->created_at,
                    'type' => $music->type,
                    'hometype' => 'music',
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch trending music.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

public function topNewSongs()
{
    try {
        // Fetch the top 20 newest songs based on the created_at field
        $newSongs = Music::orderBy('created_at', 'desc') // Order by newest first
            ->take(20) // Limit results to 20
            ->get();

        // Check if there are any songs
        if ($newSongs->isEmpty()) {
            return response()->json([
                'status' => true,
                'message' => 'No new songs found.',
                'data' => []
            ], 200);
        }

        // Return the response
        return response()->json([
            'status' => true,
            'message' => 'Top 20 new songs fetched successfully.',
            'data' => $newSongs->map(function ($song) {
                return [
                    'id' => $song->id,
                    'title' => $song->title,
                    'category_id' => $song->category_id ?? 'No Category',
                    'plays' => $song->plays,
                    'likes' => $song->likes,
                    'tag' => $song->tag,
                    'lyrics' => $song->lyrics,
                    'active' => $song->active,
                    'sound' => $song->sound,
                    'image' => $song->image ?$song->image : 'No Image',
                    'created_at' => $song->created_at,
                     'type' => $song->type,
                    'hometype' => 'music'

                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        // Handle exceptions
        return response()->json([
            'status' => false,
            'message' => 'Failed to fetch top new songs.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

private function getSongsByTag($tag)
{
    try {
        // Fetch songs with the specified tag
        $songs = Music::where('tag', 'LIKE', "%{$tag}%")
            ->inRandomOrder()
            ->take(20)
            ->get();

        if ($songs->isEmpty()) {
            return response()->json([
                'status' => true,
                'message' => "No songs found with the tag '{$tag}'.",
                'data' => []
            ], 200);
        }

        return response()->json([
            'status' => true,
            'message' => "Songs with the tag '{$tag}' fetched successfully.",
            'data' => $songs->map(function ($song) {
                return [
                    'id' => $song->id,
                    'title' => $song->title,
                    'tags' => $song->tag,
                    'category' => $song->category_id ?? 'No Category',
                                        'category_id' => $song->category_id ?? 'No Category',

                    'plays' => $song->plays,
                    'likes' => $song->likes,
                    'tag' => $song->tag,
                    'lyrics' => $song->lyrics,
                    'active' => $song->active,
                    'sound' => $song->sound,
                    'image' => $song->image ? $song->image : 'No Image',
                    'created_at' => $song->created_at,
                      'type' => $song->type,
                    'hometype' => 'music'
                ];
            }),
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => "Failed to fetch songs with the tag '{$tag}'.",
            'error' => $e->getMessage(),
        ], 500);
    }
}

public function songsByIndica()
{
    return $this->getSongsByTag('indica');
}

public function songsBySativa()
{
    return $this->getSongsByTag('sativa');
}
public function songsByHybrid()
{
    return $this->getSongsByTag('hybrid');
}

    public function popularMusic()
    {
        try {
            $popularMusic = Music::orderBy('plays', 'desc')
                ->take(10)
                ->get();

            if ($popularMusic->isEmpty()) {
                return response()->json([
                    'status' => true,
                    'message' => 'No popular music found.',
                    'data' => []
                ], 200);
            }

            return response()->json([
                'status' => true,
                'message' => 'Popular music fetched successfully.',
                'data' => $popularMusic
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Failed to fetch popular music.',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    
    
   
    
}
