<?php

namespace App\Http\Controllers;

use Illuminate\Support\Str;
use App\Models\GlobalFunction;
use App\Models\Podcast;
use App\Models\PodcastPlayLog;
use App\Models\PodcastCategory;
use App\Models\PodcastFollower;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use getID3;
   
class PodcastController extends Controller
{
public function index()
{
    $podcasts = Podcast::where("active", 1)
        ->join('podcast_categories', 'podcasts.podcast_category_id', '=', 'podcast_categories.id') // Join with podcast_categories table
        ->select(
            'podcasts.*',
            'podcast_categories.image as category_image' // Include category image
        )
        ->get();

    // Format the response data
    $data = $podcasts->map(function ($podcast) {
        
        return [
            'id' => $podcast->id,
            'title' => $podcast->title,
            'description' => $podcast->description,
            'episode' => $podcast->episode, // Convert to full URL
            'start_date' => $podcast->start_date,
            'end_date' => $podcast->end_date,
            'likes' => $podcast->likes,
            'plays' => $podcast->plays,
            'type' => $podcast->type,
            'podcast_category_id' => $podcast->podcast_category_id,
            'category_image' => $podcast->category_image, // Include category image
            'created_at' => $podcast->created_at,
            'updated_at' => $podcast->updated_at,
            'episode_number' => $podcast->episode_number ?? 1, // Default to 1 if not set
            'duration' =>$podcast->duration, // ✅ Get episode duration
        ];
    });

    return response()->json([
        'message' => 'Podcasts retrieved successfully',
        'data' => $data,
    ], 200);
}
private function getAudioDuration($filePath)
{
    if (!$filePath) {
        return 'Unknown';
    }

    $fullPath = public_path($filePath);

    if (!file_exists($fullPath)) {
        return 'Unknown';
    }

    try {
        $getID3 = new \getID3();
        $fileInfo = $getID3->analyze($fullPath);
        
        if (isset($fileInfo['playtime_seconds'])) {
            return gmdate("H:i:s", $fileInfo['playtime_seconds']); // Format duration as HH:MM:SS
        }
    } catch (\Exception $e) {
        return 'Unknown';
    }

    return 'Unknown';
}




public function store(Request $request)
{
    $validator = Validator::make($request->all(), [
        'title' => 'required|string|max:255',
        'description' => 'required|string|max:1000',
        'podcast_category_id' => 'required|integer',
        'episode' => 'required|file|mimes:mp3|max:102400',
        'start_date' => 'required|date',
        'end_date' => 'required|date',
        'likes' => 'sometimes|integer|min:0',
        'plays' => 'sometimes|integer|min:0',
        'episode_number' => 'required|integer|min:1',
    ]);

    if ($validator->fails()) {
        return response()->json([
            'message' => 'Validation failed',
            'errors' => $validator->errors(),
        ], 422);
    }

    // Save file
    $episode = $request->file('episode');
    $episodePath = GlobalFunction::saveFileAndGivePath($episode, "podcast-episodes");

    // Extract duration using getID3
    $duration = null;
    try {
        $getID3 = new getID3();
        $fileInfo = $getID3->analyze($episode->getPathname());

        if (isset($fileInfo['playtime_seconds'])) {
            $duration = gmdate("H:i:s", $fileInfo['playtime_seconds']);
        }
    } catch (\Exception $e) {
        \Log::error("Failed to get duration: " . $e->getMessage());
    }

    $podcast = Podcast::create([
        'title' => $request->title,
        'description' => $request->description,
        'podcast_category_id' => $request->podcast_category_id,
        'episode' => $episodePath,
        'start_date' => $request->start_date,
        'end_date' => $request->end_date,
        'likes' => $request->likes ?? 0,
        'plays' => $request->plays ?? 0,
         'type' => $request->type ?? 0,
        'episode_number' => $request->episode_number,
        'duration' => $duration ?? "00:00:00", // Default if extraction fails
    ]);

    return response()->json([
        'message' => 'Podcast created successfully',
        'data' => $podcast,
    ], 201);
}

public function show($id)
{
    $podcast = Podcast::where('podcasts.id', $id) // ✅ Explicit table name for 'id'
        ->join('podcast_categories', 'podcasts.podcast_category_id', '=', 'podcast_categories.id') 
        ->select('podcasts.*', 'podcast_categories.image as category_image') 
        ->first();

    if (!$podcast) {
        return response()->json([
            'status' => false,
            'message' => 'Podcast not found',
        ], 404);
    }

    return response()->json([
        'status' => true,
        'message' => 'Podcast retrieved successfully',
        'data' => [
            'id' => $podcast->id,
            'title' => $podcast->title,
            'description' => $podcast->description,
            'episode' => $podcast->episode,
            'episode_number' => $podcast->episode_number ?? 'N/A',
            'duration' => $podcast->duration ?? 'N/A',
            'start_date' => $podcast->start_date,
            'end_date' => $podcast->end_date,
            'likes' => $podcast->likes,
            'type' => $podcast->type,
            'plays' => $podcast->plays,
            'podcast_category_id' => $podcast->podcast_category_id,
            'category_image' => !empty($podcast->category_image) ? $podcast->category_image : null, 
            'created_at' => $podcast->created_at,
            'updated_at' => $podcast->updated_at,
        ],
    ], 200);
}

  public function update(Request $request, $id)
{
    $podcast = Podcast::find($id);

    if (!$podcast) {
        return response()->json(['message' => 'Podcast not found'], 404);
    }

    $validator = Validator::make($request->all(), [
        'title' => 'sometimes|required|string|max:255',
        'description' => 'sometimes|required|string|max:1000',
        'podcast_category_id' => 'sometimes|required|integer',
        'episode' => 'sometimes|file|mimes:mp3|max:102400', // Limit size to 100MB
        'start_date' => 'sometimes|required|date',
        'end_date' => 'sometimes|required|date',
        'likes' => 'sometimes|integer|min:0',
        'plays' => 'sometimes|integer|min:0',
                'type' => 'sometimes|integer|min:0',

        'episode_number' => 'sometimes|required|integer|min:1',
    ]);

    if ($validator->fails()) {
        return response()->json([
            'message' => 'Validation failed',
            'errors' => $validator->errors(),
        ], 422);
    }

    // If a new episode file is uploaded, replace the existing one and extract duration
    if ($request->hasFile('episode')) {
        GlobalFunction::deleteFile($podcast->episode);
        $episode = $request->file('episode');
        $episodePath = GlobalFunction::saveFileAndGivePath($episode, "podcast-episodes");

        // Extract duration
        $duration = null;
        try {
            $getID3 = new \getID3();
            $fileInfo = $getID3->analyze($episode->getPathname());

            if (isset($fileInfo['playtime_seconds'])) {
                $duration = gmdate("H:i:s", $fileInfo['playtime_seconds']);
            }
        } catch (\Exception $e) {
            \Log::error("Failed to get duration: " . $e->getMessage());
        }

        $podcast->episode = $episodePath;
        $podcast->duration = $duration ?? "00:00:00"; // Default if extraction fails
    }

    // Update only the provided fields
    if ($request->has('title')) {
        $podcast->title = $request->title;
    }

    if ($request->has('description')) {
        $podcast->description = $request->description;
    }

    if ($request->has('podcast_category_id')) {
        $podcast->podcast_category_id = $request->podcast_category_id;
    }

    if ($request->has('start_date')) {
        $podcast->start_date = $request->start_date;
    }

    if ($request->has('end_date')) {
        $podcast->end_date = $request->end_date;
    }

    if ($request->has('likes')) {
        $podcast->likes = $request->likes;
    }

    if ($request->has('plays')) {
        $podcast->plays = $request->plays;
    }

    if ($request->has('episode_number')) {
        $podcast->episode_number = $request->episode_number;
    }

    $podcast->save();

    return response()->json([
        'message' => 'Podcast updated successfully',
        'data' => $podcast,
    ], 200);
}


    public function destroy($id)
{
    $podcast = Podcast::find($id);
    if (!$podcast) {
        return response()->json(['message' => 'Podcast not found'], 404);
    }

    GlobalFunction::deleteFile($podcast->episode);
    $podcast->delete();

    return response()->json(['message' => 'Podcast deleted successfully'], 200);
}


    public function indexView()
    {
        $podcast_categories = PodcastCategory::all();
        return view("podcast.index", compact("podcast_categories"));
    }

    public function updateStatus(Request $request)
    {
        $podcast = Podcast::where('id', $request->id)->first();
        if ($podcast == null) {
            return response()->json([
                'status' => false,
                'message' => 'Podcast not found',
            ]);
        }

        $podcast->active = $request->active;
        $podcast->save();

        return response()->json([
            'status' => true,
            'message' => 'Podcast updated successfully',
        ]);
    }

public function playPodcast(Request $request)
{
    try {
        // Validate the request
        $validatedData = $request->validate([
            'podcast_id' => 'required|exists:podcasts,id', 
            'user_id' => 'required|exists:users,id',
        ]);

        // Find the podcast
        $podcast = Podcast::find($validatedData['podcast_id']);

        if (!$podcast) {
            return response()->json([
                'status' => false,
                'message' => 'Podcast not found.',
            ], 404);
        }

        // Increment the play count for the podcast
        $podcast->increment('plays');

        // ✅ Log the play using the PodcastPlayLog model
        PodcastPlayLog::create([
            'podcast_id' => $podcast->id,
            'user_id' => $validatedData['user_id'],
            'played_at' => now(),
        ]);

        return response()->json([
            'status' => true,
            'message' => 'Podcast played successfully.',
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to log podcast play.',
            'error' => $e->getMessage(),
        ], 500);
    }
}

public function podcastList(Request $request)
{
    $columns = [
        0 => 'id',
        1 => 'title',
        2 => 'description',
        3 => 'episode_number', // ✅ Added Episode Number
        4 => 'duration', // ✅ Added Duration
        5 => 'start_date',
        6 => 'end_date',
        7 => 'likes',
        8 => 'plays',
        9 => 'type',
    ];

    $totalData = Podcast::count();
    $limit = $request->input('length');
    $start = $request->input('start');
    $order = $columns[$request->input('order.0.column')] ?? 'id';
    $dir = $request->input('order.0.dir', 'asc');
    $search = $request->input('search.value');

    $query = Podcast::with('podcast_category');

    if (!empty($search)) {
        $query->where('title', 'LIKE', "%{$search}%");
    }

    $totalFiltered = $query->count();

    $result = $query->offset($start)
        ->limit($limit)
        ->orderBy($order, $dir)
        ->get();

    $data = [];
    foreach ($result as $item) {
        $edit = '<a href="#" class="btn edit btn-success me-3" rel="' . $item->id . '">Edit</a>';
        $delete = '<a href="#" class="btn delete btn-danger" rel="' . $item->id . '">Delete</a>';
        $action = '<span class="float-end">' . $edit . $delete . '</span>';

        // ✅ Ensure the episode file path is correct
        $episodeUrl = !empty($item->episode) ? asset($item->episode) : '';

        // ✅ Correct audio tag syntax
        $episode = !empty($episodeUrl)
            ? "<audio controls style='width:200px;'>
                <source src='" . $episodeUrl . "' type='audio/mpeg'>
               </audio>"
            : 'No Episode Available';

        $data[] = [
            'id' => $item->id,
            'title' => $item->title,
            'description' => Str::limit($item->description ?? '', 50, '...'),
            'category' => $item->podcast_category->name ?? 'No Category Found',
            'episode' => $episode, // ✅ Corrected episode field
            'episode_number' => $item->episode_number ?? 'N/A', // ✅ Add Episode Number
            'duration' => $item->duration ?? 'N/A', // ✅ Add Duration
            'start_date' => $item->start_date ?? 'N/A',
            'end_date' => $item->end_date ?? 'N/A',
            'likes' => $item->likes ?? 0,
            'plays' => $item->plays ?? 0,
                        'type' => $item->type ?? 0,

            'action' => $action,
        ];
    }

    return response()->json([
        'draw' => intval($request->input('draw')),
        'recordsTotal' => $totalData,
        'recordsFiltered' => $totalFiltered,
        'data' => $data,
    ]);
}


public function popularPodcasts()
{
    // Fetch podcast categories with episodes
    $categories = \App\Models\PodcastCategory::with('podcasts')->get();

    // Format and sort the categories based on total likes of their episodes
    $formatted = $categories->map(function ($category) {
        $totalLikes = $category->podcasts->sum('likes');
        $totalPlays = $category->podcasts->sum('plays');
        $latestEpisode = $category->podcasts->sortByDesc('created_at')->first();

        return [
            'id' => $category->id,
            'title' => $category->name,
            'description' => $category->description ?? '',
            'episode' => $latestEpisode->episode ?? '', // optional: most recent episode file
            'start_date' => $latestEpisode->start_date ?? null,
            'end_date' => $latestEpisode->end_date ?? null,
            'likes' => $totalLikes,
            'plays' => $totalPlays,
            'podcast_category_id' => $category->id,
             'category_image' => $category->image ?? 'No Image',
                          'image' => $category->image ?? 'No Image',

            'created_at' => $category->created_at,
            'updated_at' => $category->updated_at,
            'type' => 0, // no type in category? use 0
            'hometype' => 'podcasts',
        ];
    })->sortByDesc('plays')->take(10)->values();

    return response()->json([
        'message' => 'Popular podcasts retrieved successfully',
        'data' => $formatted,
    ], 200);
}


    public function podcastByCategory($category_id)
    {
        $podcasts = Podcast::with(['podcast_category'])
            ->where('podcast_category_id', $category_id)
            ->get();

        if ($podcasts->isEmpty()) {
            return response()->json([
                'message' => 'No podcasts found for the given category',
            ], 404);
        }

        return response()->json([
            'message' => 'Podcasts retrieved successfully',
            'data' => $podcasts,
        ], 200);
    }
    
 public function getEpisodesByPodcast(Request $request, $categoryId)
{
    try {
        $perPage = $request->input('per_page', 10); // Default: 10 results per page
        $page = $request->input('page', 1);

        // Fetch Podcast Category with its episodes
        $podcastCategory = PodcastCategory::where('id', $categoryId)->first();

        if (!$podcastCategory) {
            return response()->json([
                'status' => false,
                'message' => 'Podcast category not found',
                'data' => []
            ], 404);
        }

        // Fetch paginated episodes within this category
        $episodes = Podcast::where('podcast_category_id', $categoryId)
            ->orderBy('created_at', 'desc')
            ->paginate($perPage, ['*'], 'page', $page);

        return response()->json([
            'status' => true,
            'message' => 'Episodes retrieved successfully',
            'category' => [
                'id' => $podcastCategory->id,
                'name' => $podcastCategory->name,
                'description' => $podcastCategory->description ?? 'No description available',
                'image' => $podcastCategory->image ?? null,
                'tags' => explode(',', $podcastCategory->tags ?? ''), // Convert tags into an array
            ],
            'data' => $episodes // Paginated episodes list
        ], 200);
    } catch (\Exception $e) {
        return response()->json([
            'status' => false,
            'message' => 'Failed to retrieve episodes',
            'error' => $e->getMessage(),
        ], 500);
    }
}

public function toggleFollow(Request $request)
{
    $request->validate([
        'user_id' => 'required|exists:users,id',
        'podcast_id' => 'required|exists:podcast_categories,id',
    ]);

    $user_id = $request->user_id;
    $podcast_id = $request->podcast_id;

    $existingFollow = PodcastFollower::where('user_id', $user_id)
        ->where('podcast_id', $podcast_id)
        ->first();

    if ($existingFollow) {
        $existingFollow->delete();

        return response()->json([
            'status' => true,
            'message' => 'Unfollowed podcast successfully.',
            'following' => false
        ], 200);
    } else {
        PodcastFollower::create([
            'user_id' => $user_id,
            'podcast_id' => $podcast_id,
        ]);

        return response()->json([
            'status' => true,
            'message' => 'Followed podcast successfully.',
            'following' => true
        ], 200);
    }
}


    /**
     * Check if a user is following a podcast.
     */
    public function checkFollowStatus($podcast_id, $user_id)
    {
        $isFollowing = PodcastFollower::where('user_id', $user_id)
            ->where('podcast_id', $podcast_id)
            ->exists();

        return response()->json([
            'status' => true,
            'following' => $isFollowing,
        ], 200);
    }

    /**
     * Get all users following a podcast.
     */
    public function getFollowers($podcast_id)
    {
       $followersCount = PodcastFollower::where('podcast_id', $podcast_id)->count();

    return response()->json([
        'status' => true,
        'message' => 'Podcast followers count retrieved successfully.',
        'followers_count' => $followersCount,
    ], 200);
    }
}
